;;; gnus-later.el --- later support for Gnus
;; Copyright (C) 1997,98 Free Software Foundation, Inc.

;; Author: Dan Nicolaescu <dann@ics.uci.edu>
;; Keywords: news

;; This file is part of GNU Emacs.

;; GNU Emacs is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; GNU Emacs is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to the
;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;; Boston, MA 02111-1307, USA.

;;; Commentary:

;;; Code:

(require 'gnus)
(require 'gnus-sum)
(require 'message)
(require 'gnus-msg)
(require 'nnlater)
(require 'gnus-agent)
(eval-when-compile (require 'cl))

;;; Later minor mode

(defvar gnus-later-mode nil
  "Minor mode for providing summary buffers for nnlater.")

(defvar gnus-later-mode-map nil)

(unless gnus-later-mode-map
  (setq gnus-later-mode-map (make-sparse-keymap))

  (gnus-define-keys gnus-later-mode-map
    "De" gnus-later-edit-message))

(defun gnus-later-make-menu-bar ()
  (unless (boundp 'gnus-later-menu)
    (easy-menu-define
     gnus-later-menu gnus-later-mode-map ""
     '("Later"
       ["Edit" gnus-later-edit-message t]
       ["Delete later" gnus-summary-delete-article t]))))

(defun gnus-later-mode (&optional arg)
  "Minor mode for `later' summary buffers.

\\{gnus-later-mode-map}"
  (interactive "P")
  (when (eq major-mode 'gnus-summary-mode)
    (when (set (make-local-variable 'gnus-later-mode)
		  (if (null arg) (not gnus-later-mode)
		    (> (prefix-numeric-value arg) 0)))
      ;; Set up the menu.
      (when (gnus-visual-p 'later-menu 'menu)
	(gnus-later-make-menu-bar))
      (gnus-add-minor-mode 'gnus-later-mode " Later" gnus-later-mode-map)
      (gnus-run-hooks 'gnus-later-mode-hook))))

;;; Commands

(defun gnus-later-edit-message ()
  "Enter a mail/post buffer to edit and send the Later article."
  (interactive)
  (let ((article (gnus-summary-article-number)))
    (gnus-summary-mark-as-read article gnus-canceled-mark)
    (gnus-later-setup article gnus-newsgroup-name)
    (set-buffer-modified-p t)
    (save-buffer)
    (let ((gnus-verbose-backends nil))
      (gnus-request-expire-articles (list article) gnus-newsgroup-name t))
    (push
     `((lambda ()
	 (when (gnus-buffer-exists-p ,gnus-summary-buffer)
	   (save-excursion
	     (set-buffer ,gnus-summary-buffer)
	     (gnus-cache-possibly-remove-article ,article nil nil nil t)))))
     message-send-actions)))


;;; Utility functions

;;;!!!If this is byte-compiled, it fails miserably.
;;;!!!This is because `gnus-setup-message' uses uninterned symbols.
;;;!!!This has been fixed in recent versions of Emacs and XEmacs,
;;;!!!but for the time being, we'll just run this tiny function uncompiled.

(progn
(defun gnus-later-setup (narticle group)
  (gnus-setup-message 'forward
    (let ((article narticle))
      (message-mail)
      (erase-buffer)
      (if (not (gnus-request-restore-buffer article group))
	  (error "Couldn't restore the article")
	;; Insert the separator.
	(goto-char (point-min))
	(search-forward "\n\n")
	(forward-char -1)
	(insert mail-header-separator)
	(forward-line 1)
	(message-set-auto-save-file-name))))))

(provide 'gnus-later)

;;; gnus-later.el ends here
